/********************************** <lalVerbatim file="StreamInputHV">
Author: Creighton, T. D.
$Id: StreamInput.h,v 1.1 2001/08/24 22:47:59 teviet Exp $
**************************************************** </lalVerbatim> */

/********************************************************** <lalLaTeX>

\section{Header \texttt{StreamInput.h}}
\label{s:StreamInput.h}

Provides routines to read data from an open stream and store it in LAL
data structures.

\subsection*{Synopsis}
\begin{verbatim}
#include "StreamInput.h"
\end{verbatim}

\noindent This header provides prototypes for routines that construct
LAL data structures using the data from a file (or other I/O) stream.
The routines do not provide a system-level interface to create files
and open or close file streams; they simply assume that they have been
passed an open, readable stream.  Nonetheless, because they involve
I/O stream manipulation, these routines are placed in the
\verb@lalsupport@ library rather than in \verb@lal@ proper.

These routines are compartmentalized in such a way that they can
easily be converted if the LAL specification later changes the way in
which I/O streams are handled.  In partucular, the only file I/O
commands used are \verb@fgets()@ and \verb@feof()@.  Thus the upgrade
would involve only the following global changes:
\begin{enumerate}
\item Replace all occurrences of \verb@FILE *@ with the name of the
LAL I/O stream pointer type.
\item Replace all occurrences of \verb@fgets()@ and \verb@feof()@ with
equivalent LAL functions.
\end{enumerate}
In particular, there is no need to translate routines such as
\verb@fscanf()@; one should simply read data into a LAL
\verb@CHARVector@ and then use \verb@sscanf()@ to format the input.
This is the philosophy adopted in the following modules.

******************************************************* </lalLaTeX> */

#ifndef _STREAMINPUT_H
#define _STREAMINPUT_H

#include <lal/LALStdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

NRCSID(STREAMINPUTH,"$Id: StreamInput.h,v 1.1 2001/08/24 22:47:59 teviet Exp $");

/********************************************************** <lalLaTeX>
\subsection*{Error conditions}
****************************************** </lalLaTeX><lalErrTable> */
#define STREAMINPUTH_ENUL 1
#define STREAMINPUTH_EOUT 2
#define STREAMINPUTH_EMEM 3
#define STREAMINPUTH_ELEN 4

#define STREAMINPUTH_MSGENUL "Unexpected null pointer in arguments"
#define STREAMINPUTH_MSGEOUT "Output handle points to a non-null pointer"
#define STREAMINPUTH_MSGEMEM "Memory allocation error"
#define STREAMINPUTH_MSGELEN "No numbers were read"
/******************************************** </lalErrTable><lalLaTeX>

\subsection*{Types}

******************************************************* </lalLaTeX> */

/* <lalLaTeX>
\vfill{\footnotesize\input{StreamInputHV}}
</lalLaTeX> */

/* Function prototypes. */

/* <lalLaTeX>
\newpage\input{StreamVectorInputC}
</lalLaTeX> */
void
LALCHARReadVector( LALStatus  *stat, CHARVector **vector, FILE *stream );

void
LALI2ReadVector( LALStatus  *stat, INT2Vector **vector, FILE *stream );

void
LALI4ReadVector( LALStatus  *stat, INT4Vector **vector, FILE *stream );

void
LALI8ReadVector( LALStatus  *stat, INT8Vector **vector, FILE *stream );

void
LALU2ReadVector( LALStatus  *stat, UINT2Vector **vector, FILE *stream );

void
LALU4ReadVector( LALStatus  *stat, UINT4Vector **vector, FILE *stream );

void
LALU8ReadVector( LALStatus  *stat, UINT8Vector **vector, FILE *stream );

void
LALI2ReadVector( LALStatus  *stat, INT2Vector **vector, FILE *stream );

void
LALI4ReadVector( LALStatus  *stat, INT4Vector **vector, FILE *stream );

void
LALSReadVector( LALStatus  *stat, REAL4Vector **vector, FILE *stream );

void
LALDReadVector( LALStatus  *stat, REAL8Vector **vector, FILE *stream );

/* <lalLaTeX>
\newpage\input{StreamVectorSequenceInputC}
</lalLaTeX> */
void
LALCHARReadVectorSequence( LALStatus  *stat,
			   CHARVectorSequence **sequence,
			   FILE *stream );

void
LALI2ReadVectorSequence( LALStatus  *stat,
			 INT2VectorSequence **sequence,
			 FILE *stream );

void
LALI4ReadVectorSequence( LALStatus  *stat,
			 INT4VectorSequence **sequence,
			 FILE *stream );

void
LALI8ReadVectorSequence( LALStatus  *stat,
			 INT8VectorSequence **sequence,
			 FILE *stream );

void
LALU2ReadVectorSequence( LALStatus  *stat,
			 UINT2VectorSequence **sequence,
			 FILE *stream );

void
LALU4ReadVectorSequence( LALStatus  *stat,
			 UINT4VectorSequence **sequence,
			 FILE *stream );

void
LALU8ReadVectorSequence( LALStatus  *stat,
			 UINT8VectorSequence **sequence,
			 FILE *stream );

void
LALSReadVectorSequence( LALStatus  *stat,
			REAL4VectorSequence **sequence,
			FILE *stream );

void
LALDReadVectorSequence( LALStatus  *stat,
			REAL8VectorSequence **sequence,
			FILE *stream );

/* <lalLaTeX>
\newpage\input{StreamInputTestC}
</lalLaTeX> */

#ifdef __cplusplus
}
#endif

#endif /* _STREAMINPUT_H */
